from enum import IntEnum


# These are the default values from oresat-libdxwifi's tx.c/cli.h/transmitter.h
# Not all of them are being used in transmission.py, but they'll be necessary
# to know if oresat-libdxwifi's python bindings are to be integrated at a
# deeper level.

class Tx_Mode(IntEnum):
    TX_TEXT_MODE = 0
    TX_FILE_MODE = 1
    TX_STREAM_MODE = 2
    TX_DIRECTORY_MODE = 3


class DAEMON_STATUS(IntEnum):
    DAEMON_UNKNOWN_CMD = 0
    DAEMON_START = 1
    DAEMON_STOP = 2


class LOG_LEVEL(IntEnum):
    DXWIFI_LOG_OFF = 0,
    DXWIFI_LOG_FATAL = 1
    DXWIFI_LOG_ERROR = 2
    DXWIFI_LOG_WARN = 3
    DXWIFI_LOG_INFO = 4
    DXWIFI_LOG_DEBUG = 5
    DXWIFI_LOG_TRACE = 6


class RADIO_TAP_TX_FLAGS(IntEnum):
    IEEE80211_RADIOTAP_F_TX_FAIL = 0x0001
    IEEE80211_RADIOTAP_F_TX_CTS = 0x0002
    IEEE80211_RADIOTAP_F_TX_RTS = 0x0004
    IEEE80211_RADIOTAP_F_TX_NOACK = 0x0008
    IEEE80211_RADIOTAP_F_TX_NOSEQNO = 0x0010
    IEEE80211_RADIOTAP_F_TX_ORDER = 0x0020


class FCTL_TYPE(IntEnum):
    IEEE80211_FTYPE_MGMT = 0x00
    IEEE80211_FTYPE_CTL = 0x04
    IEEE80211_FTYPE_DATA = 0x08
    IEEE80211_FTYPE_EXT = 0x0c


class IEEE80211_FCTL_DATA_STYPE(IntEnum):
    IEEE80211_STYPE_DATA = 0x0000
    IEEE80211_STYPE_DATA_CFACK = 0x0010
    IEEE80211_STYPE_DATA_CFPOLL = 0x0020
    IEEE80211_STYPE_DATA_CFACKPOLL = 0x0030
    IEEE80211_STYPE_NULLFUNC = 0x0040
    IEEE80211_STYPE_CFACK = 0x0050
    IEEE80211_STYPE_CFPOLL = 0x0060
    IEEE80211_STYPE_CFACKPOLL = 0x0070
    IEEE80211_STYPE_QOS_DATA = 0x0080
    IEEE80211_STYPE_QOS_DATA_CFACK = 0x0090
    IEEE80211_STYPE_QOS_DATA_CFPOLL = 0x00A0
    IEEE80211_STYPE_QOS_DATA_CFACKPOLL = 0x00B0
    IEEE80211_STYPE_QOS_NULLFUNC = 0x00C0
    IEEE80211_STYPE_QOS_CFACK = 0x00D0
    IEEE80211_STYPE_QOS_CFPOLL = 0x00E0
    IEEE80211_STYPE_QOS_CFACKPOLL = 0x00F0


class FCTL():
    IEEE80211_PROTOCOL_VERSION = 0
    PROTOCOL_VERSION = IEEE80211_PROTOCOL_VERSION
    TYPE = FCTL_TYPE.IEEE80211_FTYPE_DATA
    STYPE = IEEE80211_FCTL_DATA_STYPE.IEEE80211_STYPE_DATA
    TO_DS = False
    FROM_DS = True
    MORE_FRAG = False
    RETRY = False
    POWER_MANAGEMENT = False
    MORE_DATA = True
    WEP = False
    ORDER = False


class TX():
    TRANSMIT_TIMEOUT = -1
    REDUNDANT_CTRL_FRAMES = 0
    ENABLE_PA = False
    RTAP_FLAGS = 0x00
    RTAP_RATE_MBPS = 1
    RTAP_TX_FLAGS = RADIO_TAP_TX_FLAGS.IEEE80211_RADIOTAP_F_TX_NOACK
    FCTL = FCTL()
    DXWIFI_DFLT_SENDER_ADDR = "0xF1:0xF1:0xF1:0xF1:0xF1:0xF1"
    ADDRESS = DXWIFI_DFLT_SENDER_ADDR


class TX_ARGS():
    TX_MODE = Tx_Mode.TX_DIRECTORY_MODE
    DXWIFI_DAEMON_CMD = DAEMON_STATUS.DAEMON_UNKNOWN_CMD
    PID_FILE = "/run/oresat-dxwifi-txd.pid"
    VERBOSITY = LOG_LEVEL.DXWIFI_LOG_INFO
    QUIET = False
    USE_SYSLOG = False
    FILE_COUNT = 0
    FILE_FILTER = "*"
    RETRANSMIT_COUNT = 0
    TRANSMIT_CURRENT_FILES = False
    LISTEN_FOR_NEW_FILES = True
    DIRWATCH_TIMEOUT = "-1"
    TX_DELAY = "0"
    FILE_DELAY = "0"
    DEVICE = "mon0"
    ERROR_RATE = "0"
    PACKET_LOSS = "0"
    TX = TX()
    CODERATE = "0.667"
